@@ -1,6 +1,6 @@
 /*  $Id$
  *
- *  Copyright 2006-2009 Enrico Tröger <enrico(dot)troeger(at)uvena(dot)de>
+ *  Copyright 2006-2011 Enrico Tröger <enrico(at)xfce(dot)org>
  *
  *  This program is free software; you can redistribute it and/or modify
  *  it under the terms of the GNU General Public License as published by
@@ -363,6 +363,13 @@ static gint process_response_content(DictData *dd, gchar **lines, gint line_no,
 }
 
 
+static void clear_query_buffer(DictData *dd)
+{
+	g_free(dd->query_buffer);
+	dd->query_buffer = NULL;
+}
+
+
 static gboolean process_server_response(DictData *dd)
 {
 	gint max_lines, i;
@@ -383,22 +390,22 @@ static gboolean process_server_response(DictData *dd)
 		case SERVER_NOT_READY:
 		{
 			dict_gui_status_add(dd, _("The server is not ready."));
-			g_free(dd->query_buffer);
+			clear_query_buffer(dd);
 			return FALSE;
 		}
 		case UNKNOWN_DATABASE:
 		{
 			dict_gui_status_add(dd,
 				_("Invalid dictionary specified. Please check your preferences."));
-			g_free(dd->query_buffer);
+			clear_query_buffer(dd);
 			return FALSE;
 		}
 	}
 
 	if (! NZV(dd->query_buffer))
 	{
-		dict_gui_status_add(dd, _("Unknown error while quering the server."));
-		g_free(dd->query_buffer);
+		dict_gui_status_add(dd, _("Unknown error while querying the server."));
+		clear_query_buffer(dd);
 		return FALSE;
 	}
 
@@ -421,7 +428,7 @@ static gboolean process_server_response(DictData *dd)
 			TAG_ERROR, TAG_BOLD, NULL);
 		dict_gui_status_add(dd, "%s", tmp);
 		g_free(tmp);
-		g_free(dd->query_buffer);
+		clear_query_buffer(dd);
 
 		/* if we had no luck searching a word, maybe we have a typo so try searching with
 		 * spell check and offer a Web search*/
@@ -453,8 +460,8 @@ static gboolean process_server_response(DictData *dd)
 	}
 	else if (strncmp("150", answer, 3) != 0 && dd->query_status != NOTHING_FOUND)
 	{
-		dict_gui_status_add(dd, _("Unknown error while quering the server."));
-		g_free(dd->query_buffer);
+		dict_gui_status_add(dd, _("Unknown error while querying the server."));
+		clear_query_buffer(dd);
 		return FALSE;
 	}
 	defs_found = atoi(answer + 4);
@@ -472,7 +479,7 @@ static gboolean process_server_response(DictData *dd)
 	max_lines = g_strv_length(lines);
 	if (lines == NULL || max_lines == 0)
 	{
-		g_free(dd->query_buffer);
+		clear_query_buffer(dd);
 		return FALSE;
 	}
 
@@ -485,7 +492,7 @@ static gboolean process_server_response(DictData *dd)
 		i = process_response_content(dd, lines, i, max_lines, header, body);
 	}
 	g_strfreev(lines);
-	g_free(dd->query_buffer);
+	clear_query_buffer(dd);
 
 	g_string_free(header, TRUE);
 	g_string_free(body, TRUE);
@@ -494,14 +501,18 @@ static gboolean process_server_response(DictData *dd)
 }
 
 
-static gchar *get_answer(DictData *dd, gint fd)
+static gint get_answer(gint fd, gchar **buffer)
 {
 	gboolean fol = TRUE;
 	gboolean sol = FALSE;
 	gboolean tol = FALSE;
-	GString *str = g_string_sized_new(100);
+	GString *str;
 	gchar c;
 	gchar ec[3];
+	gint query_status = NO_ERROR;
+
+	if (buffer != NULL)
+		str = g_string_sized_new(100);
 
 	alarm(10); /* abort after 10 seconds, there should went wrong something */
 	while (read(fd, &c, 1) > 0)
@@ -529,7 +540,9 @@ static gchar *get_answer(DictData *dd, gint fd)
 			tol = FALSE;
 		}
 
-		g_string_append_c(str, c);
+		if (buffer != NULL)
+			g_string_append_c(str, c);
+
 		if (tol)
 		{
 			if (strncmp(ec, "250", 3) == 0 || 	/* ok */
@@ -538,45 +551,50 @@ static gchar *get_answer(DictData *dd, gint fd)
 			{
 				break;
 			}
-			else if (strncmp(ec, "220", 3) == 0) /* server ready */
+			else if (strncmp(ec, "220", 3) == 0 ||	/* server ready */
+					 strncmp(ec, "221", 3) == 0)	/* good bye */
 			{
-				dd->query_status = NO_ERROR;
+				query_status = NO_ERROR;
 				break;
 			}
 			else if (strncmp(ec, "420", 3) == 0 ||
 					 strncmp(ec, "421", 3) == 0) /* server not ready (server down or shutdown) */
 			{
-				dd->query_status = SERVER_NOT_READY;
+				query_status = SERVER_NOT_READY;
 				break;
 			}
 			else if (strncmp(ec, "500", 3) == 0 ||
 					 strncmp(ec, "501", 3) == 0) /* bad command or parameters */
 			{
-				dd->query_status = BAD_COMMAND;
+				query_status = BAD_COMMAND;
 				break;
 			}
 			else if (strncmp(ec, "550", 3) == 0) /* invalid database */
 			{
-				dd->query_status = UNKNOWN_DATABASE;
+				query_status = UNKNOWN_DATABASE;
 				break;
 			}
 			else if (strncmp(ec, "552", 3) == 0) /* nothing found */
 			{
-				dd->query_status = NOTHING_FOUND;
+				query_status = NOTHING_FOUND;
 				break;
 			}
 			else if (strncmp(ec, "554", 3) == 0) /* no databases present */
 			{
-				dd->query_status = NO_DATABASES;
+				query_status = NO_DATABASES;
 				break;
 			}
 		}
 	}
 	alarm(0);
 
-	g_string_append_c(str, '\0');
+	if (buffer != NULL)
+	{
+		g_string_append_c(str, '\0');
+		*buffer = g_string_free(str, FALSE);
+	}
 
-	return g_string_free(str, FALSE);
+	return query_status;
 }
 
 
@@ -595,12 +613,13 @@ static gpointer ask_server(DictData *dd)
 	dd->query_is_running = TRUE;
 	dd->query_status = NO_CONNECTION;
 
-	g_free(get_answer(dd, fd));
+	dd->query_status = get_answer(fd, NULL);
 	if (dd->query_status == NO_ERROR)
 	{
 		/* take only the first part of the dictionary string, so let the string end at the space */
 		i = 0;
-		while (dd->dictionary[i] != ' ') i++;
+		while (dd->dictionary[i] != ' ')
+			i++;
 		dd->dictionary[i] = '\0';
 
 		g_snprintf(cmd, BUF_SIZE, "DEFINE %s \"%s\"", dd->dictionary, dd->searched_word);
@@ -609,9 +628,10 @@ static gpointer ask_server(DictData *dd)
 		/* and now, "append" again the rest of the dictionary string again */
 		dd->dictionary[i] = ' ';
 
-		dd->query_buffer = get_answer(dd, fd);
-		send_command(fd, "QUIT");
+		dd->query_status = get_answer(fd, &(dd->query_buffer));
 	}
+	send_command(fd, "QUIT");
+	get_answer(fd, NULL);
 	close(fd);
 
 	dd->query_is_running = FALSE;
@@ -623,7 +643,7 @@ static gpointer ask_server(DictData *dd)
 }
 
 
-void signal_cb(gint sig)
+static void signal_cb(gint sig)
 {
 	/* do nothing here and hope we never get called */
 }
@@ -688,7 +708,7 @@ void dict_dictd_get_information(GtkWidget *button, DictData *dd)
 
 	dd->query_status = NO_CONNECTION;
 
-	g_free(get_answer(dd, fd));
+	dd->query_status = get_answer(fd, NULL);
 	if (dd->query_status != NO_ERROR)
 	{
 		dict_show_msgbox(dd, GTK_MESSAGE_ERROR, _("Could not connect to server."));
@@ -698,12 +718,15 @@ void dict_dictd_get_information(GtkWidget *button, DictData *dd)
 	send_command(fd, "SHOW SERVER");
 
 	/* read all server output */
-	answer = buffer = get_answer(dd, fd);
+	dd->query_status = get_answer(fd, &answer);
+	buffer = answer;
 	send_command(fd, "QUIT");
+	get_answer(fd, NULL);
 	close(fd);
 
 	/* go to next line */
-	while (*buffer != '\n') buffer++;
+	while (*buffer != '\n')
+		buffer++;
 	buffer++;
 
 	if (strncmp("114", buffer, 3) != 0)
@@ -714,7 +737,9 @@ void dict_dictd_get_information(GtkWidget *button, DictData *dd)
 	}
 
 	/* go to next line */
-	while (*buffer != '\n') buffer++;
+	while (*buffer != '\n')
+		buffer++;
+
 	buffer++;
 
 	end = strstr(buffer, ".\r\n250");
@@ -780,7 +805,7 @@ void dict_dictd_get_list(GtkWidget *button, DictData *dd)
 
 	dd->query_status = NO_CONNECTION;
 
-	g_free(get_answer(dd, fd));
+	dd->query_status = get_answer(fd, NULL);
 	if (dd->query_status != NO_ERROR)
 	{
 		dict_show_msgbox(dd, GTK_MESSAGE_ERROR, _("Could not connect to server."));
@@ -790,12 +815,15 @@ void dict_dictd_get_list(GtkWidget *button, DictData *dd)
 	send_command(fd, "SHOW DATABASES");
 
 	/* read all server output */
-	answer = buffer = get_answer(dd, fd);
+	dd->query_status = get_answer(fd, &answer);
+	buffer = answer;
 	send_command(fd, "QUIT");
+	get_answer(fd, NULL);
 	close(fd);
 
 	/* go to next line */
-	while (*buffer != '\n') buffer++;
+	while (*buffer != '\n')
+		buffer++;
 	buffer++;
 
 	if (strncmp("554", buffer, 3) == 0)
@@ -805,12 +833,13 @@ void dict_dictd_get_list(GtkWidget *button, DictData *dd)
 	}
 	else if (strncmp("110", buffer, 3) != 0 && strncmp("554", buffer, 3) != 0)
 	{
-		dict_show_msgbox(dd, GTK_MESSAGE_ERROR, _("Unknown error while quering the server."));
+		dict_show_msgbox(dd, GTK_MESSAGE_ERROR, _("Unknown error while querying the server."));
 		return;
 	}
 
 	/* go to next line */
-	while (*buffer != '\n') buffer++;
+	while (*buffer != '\n')
+		buffer++;
 	buffer++;
 
 	/* clear the combo box */
