@@ -1,6 +1,6 @@
 /*  $Id$
  *
- *  Copyright 2006-2008 Enrico Tröger <enrico(dot)troeger(at)uvena(dot)de>
+ *  Copyright 2006-2012 Enrico Tröger <enrico(at)xfce(dot)org>
  *
  *  This program is free software; you can redistribute it and/or modify
  *  it under the terms of the GNU General Public License as published by
@@ -26,15 +26,16 @@
 #endif
 
 #include <string.h>
+#include <stdarg.h>
+
 #include <gtk/gtk.h>
 #include <gdk/gdkkeysyms.h>
-#include <libxfcegui4/libxfcegui4.h>
+#include <libxfce4util/libxfce4util.h>
 
 #include "common.h"
 #include "gui.h"
-#include "sexy-icon-entry.h"
-#include "searchentry.h"
 #include "inline-icon.h"
+#include "speedreader.h"
 
 
 
@@ -45,6 +46,41 @@ static gboolean entry_is_dirty = FALSE;
 
 
 /* all textview_* functions are from the gtk-demo app to get links in the textview working */
+static gchar *textview_get_hyperlink_at_iter(GtkWidget *text_view, GtkTextIter *iter, DictData *dd)
+{
+	GSList *tags = NULL, *tagp = NULL;
+	gchar *found_link = NULL;
+	gchar *result = NULL;
+
+	tags = gtk_text_iter_get_tags(iter);
+	for (tagp = tags;  tagp != NULL;  tagp = tagp->next)
+	{
+		GtkTextTag *tag = tagp->data;
+
+		found_link = g_object_get_data(G_OBJECT(tag), "link");
+		if (found_link != NULL)
+		{
+			result = g_strdup(found_link);
+			break;
+		}
+		g_object_get(G_OBJECT(tag), "name", &found_link, NULL);
+		if (found_link != NULL)
+		{
+			if (strcmp("link", found_link) == 0)
+			{
+				result = dict_get_web_query_uri(dd, dd->searched_word);
+				break;
+			}
+			g_free(found_link);
+		}
+	}
+	if (tags)
+		g_slist_free(tags);
+
+	return result;
+}
+
+
 static void textview_follow_if_link(GtkWidget *text_view, GtkTextIter *iter, DictData *dd)
 {
 	GSList *tags = NULL, *tagp = NULL;
@@ -60,13 +96,17 @@ static void textview_follow_if_link(GtkWidget *text_view, GtkTextIter *iter, Dic
 		{
 			gtk_entry_set_text(GTK_ENTRY(dd->main_entry), found_link);
 			dict_search_word(dd, found_link);
-
 			break;
 		}
 		g_object_get(G_OBJECT(tag), "name", &found_link, NULL);
 		if (found_link != NULL && strcmp("link", found_link) == 0)
 		{
-			dict_start_web_query(dd, dd->searched_word);
+			gboolean browser_started = dict_start_web_query(dd, dd->searched_word);
+			if (browser_started && dd->is_plugin)
+				gtk_widget_hide(dd->window);
+
+			gdk_window_set_cursor(gtk_text_view_get_window(
+					GTK_TEXT_VIEW(text_view), GTK_TEXT_WINDOW_TEXT), regular_cursor);
 			g_free(found_link);
 			break;
 		}
@@ -79,15 +119,14 @@ static void textview_follow_if_link(GtkWidget *text_view, GtkTextIter *iter, Dic
 static gboolean textview_key_press_event(GtkWidget *text_view, GdkEventKey *event, DictData *dd)
 {
 	GtkTextIter iter;
-	GtkTextBuffer *buffer;
 
 	switch (event->keyval)
 	{
 		case GDK_Return:
 		case GDK_KP_Enter:
 		{
-			buffer = gtk_text_view_get_buffer(GTK_TEXT_VIEW(text_view));
-			gtk_text_buffer_get_iter_at_mark(buffer, &iter, gtk_text_buffer_get_insert(buffer));
+			gtk_text_buffer_get_iter_at_mark(dd->main_textbuffer, &iter,
+				gtk_text_buffer_get_insert(dd->main_textbuffer));
 			textview_follow_if_link(text_view, &iter, dd);
 			break;
 		}
@@ -102,7 +141,6 @@ static gboolean textview_key_press_event(GtkWidget *text_view, GdkEventKey *even
 static gboolean textview_event_after(GtkWidget *text_view, GdkEvent *ev, DictData *dd)
 {
 	GtkTextIter start, end, iter;
-	GtkTextBuffer *buffer;
 	GdkEventButton *event;
 	gint x, y;
 
@@ -114,10 +152,8 @@ static gboolean textview_event_after(GtkWidget *text_view, GdkEvent *ev, DictDat
 	if (event->button != 1)
 		return FALSE;
 
-	buffer = gtk_text_view_get_buffer(GTK_TEXT_VIEW(text_view));
-
 	/* we shouldn't follow a link if the user has selected something */
-	gtk_text_buffer_get_selection_bounds(buffer, &start, &end);
+	gtk_text_buffer_get_selection_bounds(dd->main_textbuffer, &start, &end);
 	if (gtk_text_iter_get_offset(&start) != gtk_text_iter_get_offset(&end))
 		return FALSE;
 
@@ -132,13 +168,13 @@ static gboolean textview_event_after(GtkWidget *text_view, GdkEvent *ev, DictDat
 }
 
 
-static void textview_set_cursor_if_appropriate(GtkTextView *text_view, gint x, gint y)
+static void textview_set_cursor_if_appropriate(GtkTextView *view, gint x, gint y, GdkWindow *win)
 {
 	GSList *tags = NULL, *tagp = NULL;
 	GtkTextIter iter;
 	gboolean hovering = FALSE;
 
-	gtk_text_view_get_iter_at_location(text_view, &iter, x, y);
+	gtk_text_view_get_iter_at_location(view, &iter, x, y);
 
 	tags = gtk_text_iter_get_tags(&iter);
 	for (tagp = tags;  tagp != NULL;  tagp = tagp->next)
@@ -158,6 +194,7 @@ static void textview_set_cursor_if_appropriate(GtkTextView *text_view, gint x, g
 			g_free(name);
 			break;
 		}
+		g_free(name);
 	}
 
 	if (hovering != hovering_over_link)
@@ -165,11 +202,9 @@ static void textview_set_cursor_if_appropriate(GtkTextView *text_view, gint x, g
 		hovering_over_link = hovering;
 
 		if (hovering_over_link)
-			gdk_window_set_cursor(gtk_text_view_get_window(
-				text_view, GTK_TEXT_WINDOW_TEXT), hand_cursor);
+			gdk_window_set_cursor(win, hand_cursor);
 		else
-			gdk_window_set_cursor(gtk_text_view_get_window(
-				text_view, GTK_TEXT_WINDOW_TEXT), regular_cursor);
+			gdk_window_set_cursor(win, regular_cursor);
 	}
 
 	if (tags)
@@ -184,9 +219,7 @@ static gboolean textview_motion_notify_event(GtkWidget *text_view, GdkEventMotio
 	gtk_text_view_window_to_buffer_coords(GTK_TEXT_VIEW(text_view), GTK_TEXT_WINDOW_WIDGET,
 		event->x, event->y, &x, &y);
 
-	textview_set_cursor_if_appropriate(GTK_TEXT_VIEW(text_view), x, y);
-
-	gdk_window_get_pointer(text_view->window, NULL, NULL, NULL);
+	textview_set_cursor_if_appropriate(GTK_TEXT_VIEW(text_view), x, y, event->window);
 
 	return FALSE;
 }
@@ -201,12 +234,225 @@ static gboolean textview_visibility_notify_event(GtkWidget *text_view, GdkEventV
 	gtk_text_view_window_to_buffer_coords(GTK_TEXT_VIEW(text_view),
 		GTK_TEXT_WINDOW_WIDGET, wx, wy, &bx, &by);
 
-	textview_set_cursor_if_appropriate(GTK_TEXT_VIEW(text_view), bx, by);
+	textview_set_cursor_if_appropriate(GTK_TEXT_VIEW(text_view), bx, by, event->window);
 
 	return FALSE;
 }
 
 
+static gchar *textview_get_text_at_cursor(DictData *dd)
+{
+	gchar *word;
+	GtkTextIter start, end;
+
+	if (! gtk_text_buffer_get_selection_bounds(dd->main_textbuffer, &start, &end))
+	{
+		gint wx, wy, bx, by;
+
+		gdk_window_get_pointer(dd->main_textview->window, &wx, &wy, NULL);
+
+		gtk_text_view_window_to_buffer_coords(GTK_TEXT_VIEW(dd->main_textview),
+			GTK_TEXT_WINDOW_WIDGET, wx, wy, &bx, &by);
+
+		gtk_text_buffer_get_iter_at_mark(dd->main_textbuffer, &start, dd->mark_click);
+		if (! gtk_text_iter_starts_word(&start))
+			gtk_text_iter_backward_word_start(&start);
+
+		end = start;
+
+		if (gtk_text_iter_inside_word(&end))
+			gtk_text_iter_forward_word_end(&end);
+	}
+
+	word = gtk_text_buffer_get_text(dd->main_textbuffer, &start, &end, FALSE);
+
+	return word;
+}
+
+
+static void textview_popup_search_item_cb(GtkWidget *widget, DictData *dd)
+{
+	gchar *word;
+
+	word = textview_get_text_at_cursor(dd);
+
+	if (word != NULL)
+	{
+		gtk_entry_set_text(GTK_ENTRY(dd->main_entry), word);
+		dict_search_word(dd, word);
+		gtk_widget_grab_focus(dd->main_entry);
+
+		g_free(word);
+	}
+}
+
+
+static gboolean textview_is_text_at_cursor(DictData *dd)
+{
+	gchar *text;
+
+	text = textview_get_text_at_cursor(dd);
+	if (text != NULL)
+	{
+		gboolean non_empty_text = NZV(text);
+		g_free(text);
+		return non_empty_text;
+	}
+	else
+		return FALSE;
+}
+
+
+static void textview_popup_copylink_item_cb(GtkWidget *widget, DictData *dd)
+{
+	GtkClipboard *clipboard = gtk_clipboard_get(GDK_SELECTION_CLIPBOARD);
+	GtkTextIter iter;
+	gchar *hyperlink;
+
+	gtk_text_buffer_get_iter_at_mark(dd->main_textbuffer, &iter, dd->mark_click);
+	hyperlink = textview_get_hyperlink_at_iter(dd->main_textview, &iter, dd);
+	if (hyperlink != NULL)
+	{
+		gtk_clipboard_set_text(clipboard, hyperlink, -1);
+		g_free(hyperlink);
+	}
+}
+
+
+static gboolean textview_is_hyperlink_at_cursor(DictData *dd)
+{
+	GtkTextIter iter;
+	gchar *hyperlink;
+
+	gtk_text_buffer_get_iter_at_mark(dd->main_textbuffer, &iter, dd->mark_click);
+	hyperlink = textview_get_hyperlink_at_iter(dd->main_textview, &iter, dd);
+	if (hyperlink != NULL)
+	{
+		g_free(hyperlink);
+		return TRUE;
+	}
+	else
+		return FALSE;
+}
+
+
+static void textview_populate_popup_cb(GtkTextView *textview, GtkMenu *menu, DictData *dd)
+{
+	GtkWidget *search = gtk_image_menu_item_new_from_stock(GTK_STOCK_FIND, NULL);
+	GtkWidget *copy_link = gtk_image_menu_item_new_with_label(_("Copy Link"));
+	GtkWidget *sep = gtk_separator_menu_item_new();
+	GtkWidget *copy_link_image = gtk_image_new_from_stock(GTK_STOCK_COPY, GTK_ICON_SIZE_MENU);
+
+	gtk_widget_show(sep);
+	gtk_menu_shell_prepend(GTK_MENU_SHELL(menu), sep);
+
+	gtk_image_menu_item_set_image(GTK_IMAGE_MENU_ITEM(copy_link), copy_link_image);
+	gtk_widget_show(copy_link);
+	gtk_widget_set_sensitive(GTK_WIDGET(copy_link), textview_is_hyperlink_at_cursor(dd));
+	gtk_menu_shell_prepend(GTK_MENU_SHELL(menu), copy_link);
+
+	gtk_widget_show(search);
+	gtk_widget_set_sensitive(GTK_WIDGET(search), textview_is_text_at_cursor(dd));
+	gtk_menu_shell_prepend(GTK_MENU_SHELL(menu), search);
+
+	g_signal_connect(search, "activate", G_CALLBACK(textview_popup_search_item_cb), dd);
+	g_signal_connect(copy_link, "activate", G_CALLBACK(textview_popup_copylink_item_cb), dd);
+}
+
+
+static gboolean textview_button_press_cb(GtkTextView *view, GdkEventButton *event, DictData *dd)
+{
+	if (event->button == 3)
+	{
+		gint x, y;
+		GtkTextIter iter;
+
+		gtk_text_view_window_to_buffer_coords(view, GTK_TEXT_WINDOW_TEXT, event->x, event->y, &x, &y);
+		gtk_text_view_get_iter_at_location(view, &iter, x, y);
+		gtk_text_buffer_move_mark(dd->main_textbuffer, dd->mark_click, &iter);
+
+		gdk_window_set_cursor(event->window, regular_cursor);
+	}
+
+	return FALSE;
+}
+
+
+#if GTK_CHECK_VERSION(2, 12, 0)
+static gboolean textview_query_tooltip_cb(GtkWidget *widget, gint x, gint y, gboolean keyboard_mode,
+										  GtkTooltip *tooltip, DictData *dd)
+{
+	gint tx, ty;
+	GSList *tags = NULL, *tagp = NULL;
+	GtkTextIter iter;
+
+	gtk_text_view_window_to_buffer_coords(GTK_TEXT_VIEW(widget),
+		GTK_TEXT_WINDOW_WIDGET, x, y, &tx, &ty);
+
+	gtk_text_view_get_iter_at_location(GTK_TEXT_VIEW(widget), &iter, tx, ty);
+
+	tags = gtk_text_iter_get_tags(&iter);
+	for (tagp = tags;  tagp != NULL;  tagp = tagp->next)
+	{
+		GtkTextTag *tag = tagp->data;
+		gchar *name;
+
+		g_object_get(G_OBJECT(tag), "name", &name, NULL);
+		if (name != NULL && strcmp("link", name) == 0)
+		{
+			gchar *target_uri = dict_get_web_query_uri(dd, dd->searched_word);
+			gtk_tooltip_set_markup(tooltip, target_uri);
+			g_free(name);
+			g_free(target_uri);
+			return TRUE;
+		}
+		g_free(name);
+	}
+	return FALSE;
+}
+#endif
+
+
+static void textview_apply_or_remove_tags(GtkTextBuffer *buffer, const gchar *tag,
+										  GtkTextIter *start, GtkTextIter *end)
+{
+	g_return_if_fail(tag != NULL);
+
+	if (*tag == '\0')
+		gtk_text_buffer_remove_all_tags(buffer, start, end);
+	else
+		gtk_text_buffer_apply_tag_by_name(buffer, tag, start, end);
+}
+
+
+/* An empty string as tag name will clear all tags on the given word */
+void dict_gui_textview_apply_tag_to_word(GtkTextBuffer *buffer, const gchar *word,
+										 GtkTextIter *pos, const gchar *first_tag, ...)
+{
+	GtkTextIter start, end;
+	gchar *s;
+
+	g_return_if_fail(word != NULL);
+	g_return_if_fail(first_tag != NULL);
+
+	/* This is a bit ugly but works:
+	 * we search the passed word backwards from 'pos' and apply the tag to it. */
+	if (gtk_text_iter_backward_search(pos, word, GTK_TEXT_SEARCH_TEXT_ONLY, &start, &end, NULL))
+	{
+		va_list args;
+
+		textview_apply_or_remove_tags(buffer, first_tag, &start, &end);
+
+		va_start(args, first_tag);
+		for (; s = va_arg(args, gchar*), s != NULL;)
+		{
+			textview_apply_or_remove_tags(buffer, s, &start, &end);
+		}
+		va_end(args);
+	}
+}
+
+
 void dict_gui_set_panel_entry_text(DictData *dd, const gchar *text)
 {
 	if (dd->panel_entry != NULL)
@@ -226,6 +472,8 @@ void dict_gui_status_add(DictData *dd, const gchar *format, ...)
 
 	gtk_statusbar_pop(GTK_STATUSBAR(dd->statusbar), 1);
 	gtk_statusbar_push(GTK_STATUSBAR(dd->statusbar), 1, string);
+	if (dd->verbose_mode)
+		g_message("%s", string);
 }
 
 
@@ -249,33 +497,37 @@ static void entry_activate_cb(GtkEntry *entry, DictData *dd)
 }
 
 
-static void entry_icon_pressed_cb(SexyIconEntry *entry, gint icon_pos, gint button, DictData *dd)
+static void entry_icon_release_cb(GtkEntry *entry, GtkEntryIconPosition icon_pos,
+		GdkEventButton *event, DictData *dd)
 {
-	if (button != 1)
+	if (event->button != 1)
 		return;
 
-	if (icon_pos == SEXY_ICON_ENTRY_PRIMARY)
+	if (icon_pos == GTK_ENTRY_ICON_PRIMARY)
 	{
 		entry_activate_cb(NULL, dd);
 		gtk_widget_grab_focus(dd->main_entry);
 	}
-	else if (icon_pos == SEXY_ICON_ENTRY_SECONDARY)
+	else if (icon_pos == GTK_ENTRY_ICON_SECONDARY)
 	{
 		dict_gui_clear_text_buffer(dd);
 		gtk_entry_set_text(GTK_ENTRY(dd->main_entry), "");
 		dict_gui_set_panel_entry_text(dd, "");
-		dict_gui_status_add(dd, _("Ready."));
+		dict_gui_status_add(dd, _("Ready"));
 	}
 }
 
 
 static void combo_changed_cb(GtkComboBox *combo, DictData *dd)
 {
-	gchar *text = gtk_combo_box_get_active_text(combo);
+	GtkTreeIter iter;
 
-	dict_search_word(dd, text);
-
-	g_free(text);
+	if (gtk_combo_box_get_active_iter(combo, &iter))
+	{
+		gchar *text = gtk_combo_box_get_active_text(combo);
+		dict_search_word(dd, text);
+		g_free(text);
+	}
 }
 
 
@@ -327,7 +579,7 @@ static void update_search_button(DictData *dd, GtkWidget *box)
 
 	if (button == NULL)
 	{
-		button = gtk_button_new_from_stock("gtk-find");
+		button = gtk_button_new_from_stock(GTK_STOCK_FIND);
 		gtk_widget_show(button);
 		gtk_box_pack_start(GTK_BOX(box), button, FALSE, FALSE, 0);
 		g_signal_connect(button, "clicked", G_CALLBACK(entry_button_clicked_cb), dd);
@@ -340,7 +592,7 @@ static void update_search_button(DictData *dd, GtkWidget *box)
 	{
 		case DICTMODE_DICT:
 		{
-			image = gtk_image_new_from_stock("gtk-find", GTK_ICON_SIZE_BUTTON);
+			image = gtk_image_new_from_stock(GTK_STOCK_FIND, GTK_ICON_SIZE_BUTTON);
 			break;
 		}
 		case DICTMODE_WEB:
@@ -350,7 +602,7 @@ static void update_search_button(DictData *dd, GtkWidget *box)
 		}
 		case DICTMODE_SPELL:
 		{
-			image = gtk_image_new_from_stock("gtk-spell-check", GTK_ICON_SIZE_BUTTON);
+			image = gtk_image_new_from_stock(GTK_STOCK_SPELL_CHECK, GTK_ICON_SIZE_BUTTON);
 			break;
 		}
 		default:
@@ -402,7 +654,14 @@ const guint8 *dict_gui_get_icon_data(void)
 }
 
 
- static GtkWidget *create_file_menu(DictData *dd)
+static void speedreader_clicked_cb(GtkButton *button, DictData *dd)
+{
+	GtkWidget *dialog = xfd_speed_reader_new(GTK_WINDOW(dd->window), dd);
+	gtk_widget_show(dialog);
+}
+
+
+static GtkWidget *create_file_menu(DictData *dd)
 {
 	GtkWidget *menubar, *file, *file_menu, *help, *help_menu, *menu_item;
 	GtkAccelGroup *accel_group;
@@ -417,6 +676,16 @@ const guint8 *dict_gui_get_icon_data(void)
 	file_menu = gtk_menu_new();
 	gtk_menu_item_set_submenu(GTK_MENU_ITEM(file), file_menu);
 
+	menu_item = gtk_image_menu_item_new_with_mnemonic(_("Speed _Reader"));
+	gtk_image_menu_item_set_image(GTK_IMAGE_MENU_ITEM(menu_item),
+		gtk_image_new_from_stock(GTK_STOCK_JUSTIFY_CENTER, GTK_ICON_SIZE_MENU));
+	gtk_widget_add_accelerator(menu_item, "activate", accel_group,
+			GDK_r, GDK_CONTROL_MASK, GTK_ACCEL_VISIBLE);
+	g_signal_connect(menu_item, "activate", G_CALLBACK(speedreader_clicked_cb), dd);
+	gtk_container_add(GTK_CONTAINER(file_menu), menu_item);
+
+	gtk_container_add(GTK_CONTAINER(file_menu), gtk_separator_menu_item_new());
+
 	dd->pref_menu_item = gtk_image_menu_item_new_from_stock("gtk-preferences", accel_group);
 	gtk_widget_add_accelerator(dd->pref_menu_item, "activate", accel_group,
 			GDK_p, GDK_CONTROL_MASK, GTK_ACCEL_VISIBLE);
@@ -450,7 +719,6 @@ void dict_gui_finalize(DictData *dd)
 {
 	if (hand_cursor)
 		gdk_cursor_unref(hand_cursor);
-
 	if (regular_cursor)
 		gdk_cursor_unref(regular_cursor);
 }
@@ -461,11 +729,12 @@ void dict_gui_create_main_window(DictData *dd)
 	GtkWidget *main_box, *entry_box, *label_box;
 	GtkWidget *sep, *align, *scrolledwindow_results;
 	GdkPixbuf *icon;
-	GtkWidget *method_chooser, *radio, *label;
+	GtkWidget *method_chooser, *radio, *label, *button;
+	GtkAccelGroup *accel_group = gtk_accel_group_new();
 
 	dd->window = gtk_window_new(GTK_WINDOW_TOPLEVEL);
 	gtk_window_set_title(GTK_WINDOW(dd->window), _("Dictionary"));
-	gtk_window_set_default_size(GTK_WINDOW(dd->window), 500, 300);
+	gtk_window_set_default_size(GTK_WINDOW(dd->window), 580, 360);
 	gtk_widget_set_name(dd->window, "Xfce4Dict");
 
 	icon = gdk_pixbuf_new_from_inline(-1, dict_icon_data, FALSE, NULL);
@@ -479,24 +748,27 @@ void dict_gui_create_main_window(DictData *dd)
 	gtk_box_pack_start(GTK_BOX(main_box), create_file_menu(dd), FALSE, TRUE, 0);
 
 	/* entry box (label, entry, buttons) */
-	entry_box = gtk_hbox_new(FALSE, 10);
+	entry_box = gtk_hbox_new(FALSE, 5);
 	gtk_widget_show(entry_box);
 	gtk_container_set_border_width(GTK_CONTAINER(entry_box), 2);
 	gtk_box_pack_start(GTK_BOX(main_box), entry_box, FALSE, TRUE, 5);
 
 	label_box = gtk_hbox_new(FALSE, 5);
 	gtk_widget_show(label_box);
-	gtk_box_pack_start(GTK_BOX(entry_box), label_box, TRUE, TRUE, 0);
+	gtk_box_pack_start(GTK_BOX(entry_box), label_box, TRUE, TRUE, 5);
 
-	dd->main_combo = xfd_search_entry_new(_("Search term"));
+	dd->main_combo = gtk_combo_box_entry_new_text();
 	gtk_widget_show(dd->main_combo);
 	gtk_box_pack_start(GTK_BOX(label_box), dd->main_combo, TRUE, TRUE, 0);
-	g_signal_connect(dd->main_combo, "active-changed", G_CALLBACK(combo_changed_cb), dd);
+	g_signal_connect(dd->main_combo, "changed", G_CALLBACK(combo_changed_cb), dd);
 
 	dd->main_entry = gtk_bin_get_child(GTK_BIN(dd->main_combo));
+	gtk_entry_set_text(GTK_ENTRY(dd->main_entry), _("Search term"));
+	gtk_entry_set_icon_from_stock(GTK_ENTRY(dd->main_entry), GTK_ENTRY_ICON_PRIMARY, GTK_STOCK_FIND);
+	gtk_entry_set_icon_from_stock(GTK_ENTRY(dd->main_entry), GTK_ENTRY_ICON_SECONDARY, GTK_STOCK_CLEAR);
 	g_signal_connect(dd->main_entry, "changed", G_CALLBACK(entry_changed_cb), dd);
 	g_signal_connect(dd->main_entry, "activate", G_CALLBACK(entry_activate_cb), dd);
-	g_signal_connect(dd->main_entry, "icon_released", G_CALLBACK(entry_icon_pressed_cb), dd);
+	g_signal_connect(dd->main_entry, "icon-release", G_CALLBACK(entry_icon_release_cb), dd);
 	g_signal_connect(dd->main_entry, "button-press-event", G_CALLBACK(entry_button_press_cb), dd);
 
 	update_search_button(dd, entry_box);
@@ -506,16 +778,27 @@ void dict_gui_create_main_window(DictData *dd)
 	gtk_alignment_set_padding(GTK_ALIGNMENT(align), 0, 0, 10, 0);
 	gtk_widget_show(align);
 	gtk_container_add(GTK_CONTAINER(align), gtk_label_new(""));
-	gtk_box_pack_start(GTK_BOX(entry_box), align, FALSE, FALSE, 0);
+	gtk_box_pack_start(GTK_BOX(entry_box), align, FALSE, FALSE, 5);
 
-	dd->close_button = gtk_button_new_from_stock((dd->is_plugin) ? "gtk-close" : "gtk-quit");
-	gtk_widget_show(dd->close_button);
-	gtk_box_pack_end(GTK_BOX(entry_box), dd->close_button, FALSE, FALSE, 2);
-
-	/* insert it here and it will(hopefully) be placed before the Close button */
 	sep = gtk_vseparator_new();
 	gtk_widget_show(sep);
-	gtk_box_pack_end(GTK_BOX(entry_box), sep, FALSE, FALSE, 5);
+	gtk_box_pack_start(GTK_BOX(entry_box), sep, FALSE, FALSE, 2);
+
+	button = gtk_button_new_with_mnemonic(_("Speed _Reader"));
+	gtk_button_set_image(GTK_BUTTON(button),
+		gtk_image_new_from_stock(GTK_STOCK_JUSTIFY_CENTER, GTK_ICON_SIZE_MENU));
+	g_signal_connect(button, "clicked", G_CALLBACK(speedreader_clicked_cb), dd);
+	gtk_widget_show(button);
+	gtk_box_pack_start(GTK_BOX(entry_box), button, FALSE, FALSE, 2);
+
+	sep = gtk_vseparator_new();
+	gtk_widget_show(sep);
+	gtk_box_pack_start(GTK_BOX(entry_box), sep, FALSE, FALSE, 2);
+
+	dd->close_button = gtk_button_new_from_stock(
+		(dd->is_plugin) ? GTK_STOCK_CLOSE : GTK_STOCK_QUIT);
+	gtk_widget_show(dd->close_button);
+	gtk_box_pack_end(GTK_BOX(entry_box), dd->close_button, FALSE, FALSE, 0);
 
 	/* search method chooser */
 	method_chooser = gtk_hbox_new(FALSE, 0);
@@ -526,19 +809,21 @@ void dict_gui_create_main_window(DictData *dd)
 	gtk_widget_show(label);
 	gtk_box_pack_start(GTK_BOX(method_chooser), label, FALSE, FALSE, 6);
 
-	radio = gtk_radio_button_new_with_mnemonic(NULL, _("_Dict"));
+	radio = gtk_radio_button_new_with_mnemonic(NULL, _("_Dictionary Server"));
 	gtk_widget_show(radio);
 	gtk_toggle_button_set_active(GTK_TOGGLE_BUTTON(radio), (dd->mode_in_use == DICTMODE_DICT));
 	g_signal_connect(radio, "toggled", G_CALLBACK(search_mode_dict_toggled), dd);
 	gtk_box_pack_start(GTK_BOX(method_chooser), radio, FALSE, FALSE, 6);
 
-	radio = gtk_radio_button_new_with_mnemonic_from_widget(GTK_RADIO_BUTTON(radio), _("_Web"));
+	radio = gtk_radio_button_new_with_mnemonic_from_widget(GTK_RADIO_BUTTON(radio), _("_Web Service"));
+	dd->radio_button_web = radio;
+	gtk_widget_set_sensitive(dd->radio_button_web, NZV(dd->web_url));
 	gtk_toggle_button_set_active(GTK_TOGGLE_BUTTON(radio), (dd->mode_in_use == DICTMODE_WEB));
 	g_signal_connect(radio, "toggled", G_CALLBACK(search_mode_web_toggled), dd);
 	gtk_widget_show(radio);
 	gtk_box_pack_start(GTK_BOX(method_chooser), radio, FALSE, FALSE, 6);
 
-	radio = gtk_radio_button_new_with_mnemonic_from_widget(GTK_RADIO_BUTTON(radio), _("_Spell Check"));
+	radio = gtk_radio_button_new_with_mnemonic_from_widget(GTK_RADIO_BUTTON(radio), _("_Spell Checker"));
 	gtk_toggle_button_set_active(GTK_TOGGLE_BUTTON(radio), (dd->mode_in_use == DICTMODE_SPELL));
 	g_signal_connect(radio, "toggled", G_CALLBACK(search_mode_spell_toggled), dd);
 	gtk_widget_show(radio);
@@ -563,19 +848,30 @@ void dict_gui_create_main_window(DictData *dd)
 	gtk_text_view_set_wrap_mode(GTK_TEXT_VIEW(dd->main_textview), GTK_WRAP_WORD);
 	dd->main_textbuffer = gtk_text_view_get_buffer(GTK_TEXT_VIEW(dd->main_textview));
 	gtk_text_buffer_create_tag(dd->main_textbuffer,
-			"bold",
+			TAG_HEADING,
 			"weight", PANGO_WEIGHT_BOLD,
-			"style", PANGO_STYLE_ITALIC,
-			"indent", 10,
 			"pixels-below-lines", 5, NULL);
-	dd->phon_tag = gtk_text_buffer_create_tag(dd->main_textbuffer,
-			"phonetic",
+	gtk_text_buffer_create_tag(dd->main_textbuffer,
+			TAG_BOLD,
+			"weight", PANGO_WEIGHT_BOLD,
+			"indent", 10,
 			"style", PANGO_STYLE_ITALIC,
-			"foreground-gdk", dd->phon_color, NULL);
+			"pixels-below-lines", 5, NULL);
+	dd->error_tag = gtk_text_buffer_create_tag(dd->main_textbuffer,
+			TAG_ERROR,
+			"style", PANGO_STYLE_ITALIC,
+			"foreground-gdk", dd->color_incorrect, NULL);
+	dd->success_tag = gtk_text_buffer_create_tag(dd->main_textbuffer,
+			TAG_SUCCESS,
+			"foreground-gdk", dd->color_correct, NULL);
+	dd->phon_tag = gtk_text_buffer_create_tag(dd->main_textbuffer,
+			TAG_PHONETIC,
+			"style", PANGO_STYLE_ITALIC,
+			"foreground-gdk", dd->color_phonetic, NULL);
 	dd->link_tag = gtk_text_buffer_create_tag(dd->main_textbuffer,
-			"link",
+			TAG_LINK,
 			"underline", PANGO_UNDERLINE_SINGLE,
-			"foreground-gdk", dd->link_color, NULL);
+			"foreground-gdk", dd->color_link, NULL);
 
 	/* support for links (cross-references) for dictd responses */
 	{
@@ -591,6 +887,22 @@ void dict_gui_create_main_window(DictData *dd)
 		g_signal_connect(dd->main_textview, "visibility-notify-event",
 			G_CALLBACK(textview_visibility_notify_event), NULL);
 	}
+	/* support for 'Search' and 'Copy Link' menu items in the textview popup menu */
+	{
+		GtkTextIter start;
+		gtk_text_buffer_get_bounds(dd->main_textbuffer, &start, &start);
+		dd->mark_click = gtk_text_buffer_create_mark(dd->main_textbuffer, NULL, &start, TRUE);
+
+		g_signal_connect(dd->main_textview, "button-press-event",
+			G_CALLBACK(textview_button_press_cb), dd);
+		g_signal_connect(dd->main_textview, "populate-popup",
+			G_CALLBACK(textview_populate_popup_cb), dd);
+	}
+	/* tooltips */
+#if GTK_CHECK_VERSION(2, 12, 0)
+	gtk_widget_set_has_tooltip(dd->main_textview, TRUE);
+	g_signal_connect(dd->main_textview, "query-tooltip", G_CALLBACK(textview_query_tooltip_cb), dd);
+#endif
 
 	gtk_widget_show(dd->main_textview);
 	gtk_container_add(GTK_CONTAINER(scrolledwindow_results), dd->main_textview);
@@ -614,6 +926,9 @@ void dict_gui_create_main_window(DictData *dd)
 		if (dd->geometry[4] == 1)
 			gtk_window_maximize(GTK_WINDOW(dd->window));
 	}
+	/* quit on Escape */
+	gtk_widget_add_accelerator(dd->close_button, "clicked", accel_group, GDK_Escape, 0, 0);
+	gtk_window_add_accel_group(GTK_WINDOW(dd->window), accel_group);
 }
 
 
@@ -645,7 +960,7 @@ void dict_gui_about_dialog(GtkWidget *widget, DictData *dd)
 		"destroy-with-parent", TRUE,
 		"authors", authors,
 		"comments", _("A client program to query different dictionaries."),
-		"copyright", _("Copyright \302\251 2006-2008 Enrico Tröger"),
+		"copyright", _("Copyright \302\251 2006-2013 Enrico Tröger"),
 		"website", "http://goodies.xfce.org/projects/applications/xfce4-dict",
 		"logo", logo,
 		"translator-credits", _("translator-credits"),
