@@ -1,6 +1,6 @@
 /*  $Id$
  *
- *  Copyright 2008-2009 Enrico Tröger <enrico(dot)troeger(at)uvena(dot)de>
+ *  Copyright 2008-2011 Enrico Tröger <enrico(at)xfce(dot)org>
  *
  *  This program is free software; you can redistribute it and/or modify
  *  it under the terms of the GNU General Public License as published by
@@ -22,6 +22,6 @@
 #define POPUP_PLUGIN_H 1
 
 
-gboolean dict_find_panel_plugin(gboolean focus_panel_entry, const gchar *text);
+gboolean dict_find_panel_plugin(gchar flags, const gchar *text);
 
 #endif
