@@ -1,6 +1,6 @@
 /*  $Id$
  *
- *  Copyright 2006-2009 Enrico Tröger <enrico(dot)troeger(at)uvena(dot)de>
+ *  Copyright 2006-2012 Enrico Tröger <enrico(at)xfce(dot)org>
  *
  *  This program is free software; you can redistribute it and/or modify
  *  it under the terms of the GNU General Public License as published by
@@ -76,7 +76,7 @@ static void close_button_clicked(GtkWidget *button, DictData *dd)
 }
 
 
-void pref_dialog_activated(GtkMenuItem *menuitem, DictData *dd)
+static void pref_dialog_activated(GtkMenuItem *menuitem, DictData *dd)
 {
 	GtkWidget *dlg;
 
@@ -142,7 +142,7 @@ gint main(gint argc, gchar *argv[])
 	if (show_version)
 	{
 		g_print("%s %s (Xfce %s)\n\n", PACKAGE, PACKAGE_VERSION, xfce_version_string());
-		g_print("%s\n", "Copyright (c) 2006-2009");
+		g_print("%s\n", "Copyright (c) 2006-2012");
 		g_print("\tEnrico Tröger <enrico@xfce.org>\n\n");
 		g_print(_("Please report bugs to <%s>."), PACKAGE_BUGREPORT);
 		g_print("\n");
@@ -154,11 +154,7 @@ gint main(gint argc, gchar *argv[])
 
 	if (use_clipboard)
 	{
-		search_text = gtk_clipboard_wait_for_text(gtk_clipboard_get(
-						gdk_atom_intern("PRIMARY", FALSE)));
-		if (! search_text)
-			search_text = gtk_clipboard_wait_for_text(gtk_clipboard_get(
-							gdk_atom_intern("CLIPBOARD", FALSE)));
+		search_text = dict_get_clipboard_contents();
 	}
 	else
 	{
